@@ -1,5 +1,6 @@
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <ctype.h>
 #include <errno.h>
 #include <time.h>
@@ -29,7 +30,9 @@
 static gchar *FORMATED_NUMBER_STRING(gulong i);
 static gchar *TIME_LAPSED_STRING(gulong dt);
 static void PRINT_ERR(const gchar *path, const gchar *mesg);
-static void PRINT_SCANNING(const gchar *path);
+static void PRINT_SCANNING_FILE(
+	const gulong i, const gulong n, const gchar *path
+);
 static void PRINT_SCANNING_DIR(const gchar *path);
 static void PRINT_REPORT(
 	gulong time_start, gulong time_end,
@@ -38,7 +41,7 @@ static void PRINT_REPORT(
 	gulong files_scanned 
 );
 
-static gchar *AVScanCopyShortenString(const gchar *s, gint max);
+static gchar *AVScanCopyShortenString(const gchar *s, const gint m);
 
 
 #if defined(HAVE_CLAMAV)
@@ -49,9 +52,10 @@ typedef struct {
 	gint		database_entries;	/* Entries in the database */
 	struct cl_stat	database_stat;		/* Database stats, for
 						 * checking database changes */
-	gulong		files_scanned,
+	gulong		files_scanned,		/* Files scanned so far */
 			size_scanned_mb,
-			infected_files;
+			infected_files,
+			total_files;		/* Total files to be scanned */
 } ClamAVData;
 static ClamAVData *AVScanOPClamAVInit(const gchar *db_path);
 static void AVScanOPClamAVShutdown(ClamAVData *d);
@@ -61,19 +65,34 @@ static gint AVScanOPClamAVScanFile(
 );
 static gint AVScanOPClamAVScanDir(
 	ClamAVData *d, const gchar *path,
-	gboolean recursive,
-	gboolean executables_only,
+	const gboolean recursive,
+	const gboolean executables_only,
+	const gboolean ignore_links,
 	gint *stop_count
 );
 #endif	/* HAVE_CLAMAV */
 
 
+static gulong AVScanOPCalculateTotalFilesInDirIterate(
+	const gchar *path,
+	const gboolean recursive,
+	const gboolean executables_only,
+	const gboolean ignore_links
+);
+static gulong AVScanOPCalculateTotalFiles(
+	GList *paths_list,
+	const gboolean recursive,
+	const gboolean executables_only,
+	const gboolean ignore_links
+);
+
 gchar *AVScanGetAVEngineNameVersionString(void);
 gint AVScanOP(
 	const gchar *db_path,
-	GList *path_list,
-	gboolean recursive,
-	gboolean executables_only,
+	GList *paths_list,
+	const gboolean recursive,
+	const gboolean executables_only,
+	const gboolean ignore_links,
 	gint *stop_count
 );
 
@@ -247,17 +266,15 @@ static void PRINT_ERR(const gchar *path, const gchar *mesg)
  *	This is intended to indicate and have the monitoring process
  *	display this message on its status.
  */
-static void PRINT_SCANNING(const gchar *path)
+static void PRINT_SCANNING_FILE(
+	const gulong i, const gulong n, const gchar *path
+)
 {
-	gchar *dpath = AVScanCopyShortenString(path, 60);
-	g_print("Scanning: %s\n", dpath);
-	g_free(dpath);
+	g_print("Scanning file: \"%s\" %ld %ld\n", path, i, n);
 }
 static void PRINT_SCANNING_DIR(const gchar *path)
 {
-	gchar *dpath = AVScanCopyShortenString(path, 50);
-	g_print("Scanning directory: %s\n", dpath);
-	g_free(dpath);
+	g_print("Scanning directory: \"%s\"\n", path);
 }
 
 /*
@@ -290,11 +307,9 @@ static void PRINT_REPORT(
 		);
 
 	/* Remove tailing newline character from time_start_s */
-	for(s = time_start_s; *s != '\0'; s++)
-	{
-	    if(*s == '\n')
-		*s = '\0';
-	}
+	s = strchr(time_start_s, '\n');
+	if(s != NULL)
+	    *s = '\0';
 
 	/* Note that all ';' characters indicate newlines for the
 	 * monitoring process that reads this
@@ -345,7 +360,7 @@ Data Scanned: %s mb;\
  *	characters with appropriate shortened notation where 
  *	appropriate.
  */
-static gchar *AVScanCopyShortenString(const gchar *s, gint max)
+static gchar *AVScanCopyShortenString(const gchar *s, const gint m)
 {
 	gint len;
 
@@ -353,10 +368,10 @@ static gchar *AVScanCopyShortenString(const gchar *s, gint max)
 	    return(NULL);
 
 	len = STRLEN(s);
-	if((len > max) && (max > 3))
+	if((len > m) && (m > 3))
 	{
 	    /* Need to shorten string */
-	    gint i = len - max + 3;
+	    gint i = len - m + 3;
 
 	    return(g_strdup_printf(
 		"...%s", (const gchar *)(s + i)
@@ -381,10 +396,16 @@ static ClamAVData *AVScanOPClamAVInit(const gchar *db_path)
 	ClamAVData *d = (ClamAVData *)g_malloc0(
 	    sizeof(ClamAVData)
 	);
+	if(d == NULL)  
+	{
+	    g_print("Memory allocation error\n");
+	    return(NULL);
+	}
 
 	d->files_scanned = 0l;
 	d->size_scanned_mb = 0l;
 	d->infected_files = 0l;
+	d->total_files = 0l;
 
 	/* If no virus database was specified then get the default
 	 * virus database set by ClamAV
@@ -398,6 +419,12 @@ static ClamAVData *AVScanOPClamAVInit(const gchar *db_path)
 	    return(NULL);
 	}
 	d->database_path = STRDUP(db_path);
+	if(d->database_path == NULL)
+	{
+	    g_print("Memory allocation error\n");
+	    g_free(d);
+	    return(NULL);
+	}
 
 	/* Stat and load the virus database */ 
 	s = AVScanCopyShortenString(d->database_path, 50);
@@ -412,6 +439,13 @@ static ClamAVData *AVScanOPClamAVInit(const gchar *db_path)
 #endif
 	{
 	    d->database_dir = STRDUP(d->database_path);
+	    if(d->database_dir == NULL)
+	    {
+		g_print("Memory allocation error\n");
+		g_free(d->database_path);
+		g_free(d);
+		return(NULL);
+	    }
 
 	    memset(&d->database_stat, 0x00, sizeof(struct cl_stat));
 	    cl_statinidir(d->database_dir, &d->database_stat);     
@@ -423,6 +457,13 @@ static ClamAVData *AVScanOPClamAVInit(const gchar *db_path)
 	else
 	{
 	    d->database_dir = g_dirname(d->database_path);
+	    if(d->database_dir == NULL)  
+	    {
+		g_print("Memory allocation error\n");
+		g_free(d->database_path);
+		g_free(d);
+		return(NULL);
+	    }
 
 	    memset(&d->database_stat, 0x00, sizeof(struct cl_stat));
 	    cl_statinidir(d->database_dir, &d->database_stat);
@@ -433,7 +474,9 @@ static ClamAVData *AVScanOPClamAVInit(const gchar *db_path)
 	}
 	if(status || (d->database == NULL) || (d->database_entries <= 0))
 	{
-	    g_print("Failed to load the virus database\n");
+	    g_print(
+"Failed to load the virus database\n"
+	    );
 	    cl_statfree(&d->database_stat);
 	    g_free(d->database_path);
 	    g_free(d->database_dir);
@@ -444,7 +487,9 @@ static ClamAVData *AVScanOPClamAVInit(const gchar *db_path)
 	/* Build the virus database */
 	if(cl_build(d->database))
 	{
-	    g_print("Failed to build the virus database\n");
+	    g_print(
+"Failed to build the virus database\n"
+	    );
 	    cl_statfree(&d->database_stat);
 	    g_free(d->database_path);
 	    g_free(d->database_dir);
@@ -490,13 +535,31 @@ static gint AVScanOPClamAVScanFile(
 	gulong size_mb = 0l;
 	struct cl_limits limits;
 
-	PRINT_SCANNING(path);
+	PRINT_SCANNING_FILE(d->files_scanned, d->total_files, path);
 
-	/* Open file for reading */
+	/* Open the file for reading */
 	fd = open(path, O_RDONLY);
 	if(fd < 0)
 	{
-	    PRINT_ERR(path, "Unable to open file");
+	    const gint error_code = errno;
+	    gchar *error_msg = STRDUP(strerror((int)error_code));
+	    if(error_msg != NULL)
+	    {
+		gchar *s;
+
+		*error_msg = toupper(*error_msg);
+		s = g_strdup_printf(
+		    "%s",
+		    error_msg
+		);
+		PRINT_ERR(path, s);
+		g_free(s);
+		g_free(error_msg);
+	    }
+	    else
+	    {
+		PRINT_ERR(path, "Unable to open the file for reading");
+	    }
 	    return(1);
 	}
 
@@ -529,20 +592,6 @@ static gint AVScanOPClamAVScanFile(
  	options = 0;
 #endif
 
-#if 0
-CL_SCAN_RAW
-CL_SCAN_ARCHIVE
-CL_SCAN_MAIL
-CL_SCAN_DISABLERAR
-CL_SCAN_OLE2
-CL_SCAN_BLOCKENCRYPTED
-CL_SCAN_HTML
-CL_SCAN_PE
-CL_SCAN_BLOCKBROKEN
-CL_SCAN_MAILURL
-CL_SCAN_BLOCKMAX
-#endif   
-
 	/* Scan this file */
 	status = cl_scandesc(
 	    fd,
@@ -584,7 +633,7 @@ CL_SCAN_BLOCKMAX
 	/* Other error */
 	else
 	{
-	    gchar *s, *error_msg = STRDUP(cl_strerror(status));
+	    gchar *s, *error_msg = STRDUP(cl_strerror((int)status));
 	    if(error_msg != NULL)
 	    {
 		*error_msg = toupper(*error_msg);
@@ -605,8 +654,9 @@ CL_SCAN_BLOCKMAX
  */
 static gint AVScanOPClamAVScanDir(
 	ClamAVData *d, const gchar *path,
-	gboolean recursive,
-	gboolean executables_only,
+	const gboolean recursive,
+	const gboolean executables_only,
+	const gboolean ignore_links,
 	gint *stop_count
 )
 {
@@ -619,7 +669,25 @@ static gint AVScanOPClamAVScanDir(
 	DIR *dp = opendir(path);
 	if(dp == NULL)
 	{
-	    PRINT_ERR(path, "Unable to scan directory");
+	    const gint error_code = errno;
+	    gchar *error_msg = STRDUP(strerror((int)error_code));
+	    if(error_msg != NULL)
+	    {
+		gchar *s;
+
+		*error_msg = toupper(*error_msg);
+		s = g_strdup_printf(
+		    "%s",
+		    error_msg
+		);
+		PRINT_ERR(path, s);
+		g_free(s);
+		g_free(error_msg);
+	    }
+	    else
+	    {
+		PRINT_ERR(path, "Unable to open the directory");
+	    }
 	    return(1);
 	}
 
@@ -679,17 +747,76 @@ static gint AVScanOPClamAVScanDir(
 	    )
 		continue;
 
-	    /* Get full path to child object */
+	    /* Get the full path to the child object */
 	    child = g_strdup_printf(
 		"%s%c%s",
 		path, DIR_DELIMINATOR, s
 	    );
+	    if(child == NULL)
+		continue;
 
-	    /* Get stats for this child object */
+	    /* Get this object's local stats */
+	    if(lstat(child, &stat_buf))
+	    {
+		const gint error_code = (gint)errno;
+		gchar *error_msg = STRDUP(g_strerror((int)error_code));
+		if(error_msg != NULL)
+		{
+		    *error_msg = toupper(*error_msg);
+		    PRINT_ERR(child, error_msg);
+		    g_free(error_msg);
+		}
+		else
+		{
+		    PRINT_ERR(child, "Unable to obtain local stats");
+		}
+		g_free(child);
+		continue;
+	    }
+
+	    m = stat_buf.st_mode;
+
+	    /* Link? */
+#ifdef S_ISLNK
+	    if(S_ISLNK(m))
+#else
+	    if(FALSE)
+#endif
+	    {
+		gchar *dest;
+
+		/* Ignore links? */
+		if(ignore_links)
+		{
+		    g_free(child);
+		    continue;
+		}
+
+		/* Get the destination value and see if it is a
+		 * reference to the current directory or parent
+		 * directory
+		 */
+		dest = (gchar *)GetAllocLinkDest(child);
+		if(dest != NULL)
+		{
+		    if(!g_strcasecmp(dest, ".") ||
+		       !g_strcasecmp(dest, "..")
+		    )
+		    {
+			/* Skip this to prevent infinite recursion */
+			g_free(child);
+			g_free(dest);
+			continue;
+		    }
+		    g_free(dest);
+		}
+	    }
+
+	    /* Get this object's destination stats */
 	    if(stat(child, &stat_buf))
 	    {
-		const gint errnum = (gint)errno;
-		gchar *error_msg = STRDUP(g_strerror(errnum));
+		const gint error_code = (gint)errno;
+		gchar *error_msg = STRDUP(g_strerror((int)error_code));
 		if(error_msg != NULL)
 		{
 		    *error_msg = toupper(*error_msg);
@@ -706,9 +833,7 @@ static gint AVScanOPClamAVScanDir(
 
 	    m = stat_buf.st_mode;
 
-	    /* Is this child object a directory and should we recurse
-	     * into it?
-	     */
+	    /* Directory and recursive? */
 #ifdef S_ISDIR
 	    if(S_ISDIR(m) && recursive)
 #else
@@ -716,14 +841,16 @@ static gint AVScanOPClamAVScanDir(
 #endif
 	    {
 		status2 = AVScanOPClamAVScanDir(
-		    d, child, recursive, executables_only,
+		    d, child,
+		    recursive, executables_only, ignore_links,
 		    stop_count
 		);
 		if((status2 != 0) && (status == 0))
 		    status = status2;
 	    }
+	    /* Executables only or all? */
 	    else if(executables_only ?
-		((m & S_IXUSR) || (m & S_IXGRP) || (m & S_IXOTH)) : TRUE
+	((m & S_IXUSR) || (m & S_IXGRP) || (m & S_IXOTH)) : TRUE
 	    )
 	    {
 		status2 = AVScanOPClamAVScanFile(d, child, stop_count);
@@ -745,6 +872,241 @@ static gint AVScanOPClamAVScanDir(
 #endif	/* HAVE_CLAMAV */
 
 
+/*
+ *	Calculates the total files to be scanned in the directory
+ *	specified by path.
+ *
+ *	If recursive is TRUE then files to be scanned in subdirectories
+ *	of the specified directory will be counted.
+ *
+ *	If executables_only is TRUE then only files that are set
+ *	executable will be counted.
+ */
+static gulong AVScanOPCalculateTotalFilesInDirIterate(
+	const gchar *path,
+	const gboolean recursive,
+	const gboolean executables_only,
+	const gboolean ignore_links
+)
+{
+	mode_t m;
+	gulong subtotal = 0l;
+	gchar *child;
+	const char *s;
+	struct dirent *dent;
+	struct stat stat_buf;
+	DIR *dp = opendir(path);
+	if(dp == NULL)
+	    return(subtotal);
+
+	while(TRUE)
+	{
+	    dent = readdir(dp);
+	    if(dent == NULL)
+		break;
+
+	    s = dent->d_name;
+
+	    /* Skip special directory notations */
+	    if(!g_strcasecmp(s, ".") ||
+	       !g_strcasecmp(s, "..")
+	    )
+		continue;
+
+	    /* Get the full path to the child object */
+	    child = g_strdup_printf(
+		"%s%c%s",
+		path, DIR_DELIMINATOR, s   
+	    );
+	    if(child == NULL)
+		continue;
+
+	    /* Get the object's local stats */
+	    if(lstat(child, &stat_buf))
+	    {
+		/* Unable to get the local stats, do not count this */
+		g_free(child);
+		continue;
+	    }
+
+	    m = stat_buf.st_mode;
+
+#ifdef S_ISLNK
+	    /* Link? */
+	    if(S_ISLNK(m))
+#else
+	    if(FALSE)
+#endif
+	    {
+		gchar *dest;
+
+		/* Ignore links? */
+		if(ignore_links)
+		{
+		    /* This is a link and it is ignored, do not count this */
+		    g_free(child);
+		    continue;
+		}
+
+		/* Get the destination value and see if it is a
+		 * reference to the current directory or parent
+		 * directory
+		 */
+		dest = (gchar *)GetAllocLinkDest(child);
+		if(dest != NULL)
+		{
+		    if(!g_strcasecmp(dest, ".") ||
+		       !g_strcasecmp(dest, "..")
+		    )
+		    {
+			/* Skip this to prevent infinite recursion */
+			g_free(child);
+			g_free(dest);
+			continue;
+		    }
+		    g_free(dest);
+		}
+	    }
+
+	    /* Get this object's destination stats */
+	    if(stat(child, &stat_buf))  
+	    {
+		/* Unable to get the destination stats, do not count this */
+		g_free(child);
+		continue;
+	    }
+
+	    m = stat_buf.st_mode;
+
+	    /* Directory and recursive? */
+#ifdef S_ISDIR
+	    if(S_ISDIR(m) && recursive)
+#else
+	    if(FALSE)
+#endif
+	    {
+		subtotal += AVScanOPCalculateTotalFilesInDirIterate(
+		    child,
+		    recursive, executables_only, ignore_links
+		);
+	    }
+	    /* Executable only or all? */
+	    else if(executables_only ?
+	((m & S_IXUSR) || (m & S_IXGRP) || (m & S_IXOTH)) : TRUE
+	    )
+	    {
+		subtotal++;
+	    }
+
+	    g_free(child);
+	} 
+
+	closedir(dp);
+
+	return(subtotal);
+}
+
+/*
+ *	Calculates the total number of files to be scanned from the
+ *	specified list of paths.
+ *
+ *	If a file is not accessable to be scanned in the specified
+ *	path then it will be excluded from the returned total.
+ */
+static gulong AVScanOPCalculateTotalFiles(    
+	GList *paths_list,
+	const gboolean recursive,
+	const gboolean executables_only,
+	const gboolean ignore_links
+)
+{
+	gulong total = 0l;
+	mode_t m;
+	struct stat stat_buf;
+	const gchar *path;
+	GList *glist;
+
+	if(paths_list == NULL)
+	    return(total);
+
+	/* Iterate through each object in the paths list */
+	for(glist = paths_list; glist != NULL; glist = g_list_next(glist))
+	{
+	    path = (const gchar *)glist->data;
+	    if(STRISEMPTY(path))
+		continue;
+
+	    /* Get this object's local stats */
+	    if(lstat(path, &stat_buf))
+	    {
+		/* Unable to get local stats, do not count this */
+		continue;
+	    }
+
+	    m = stat_buf.st_mode;
+
+	    /* Link? */
+#ifdef S_ISLNK
+	    if(S_ISLNK(m))
+#else
+	    if(FALSE)
+#endif
+	    {
+		/* Links ignored? */
+		if(ignore_links)
+		{
+		    /* This is a link and links are ignored, do not count this */
+		    continue;
+		}
+
+		/* Get this link's destination stats */
+		if(stat(path, &stat_buf))
+		{
+		    /* Unable to get destination stats, do not count this */
+		    continue;
+		}
+
+		m = stat_buf.st_mode;
+
+#ifdef S_ISDIR
+		if(S_ISDIR(m))
+#else
+		if(FALSE)
+#endif
+		    total += AVScanOPCalculateTotalFilesInDirIterate(
+			path,
+			recursive, executables_only, ignore_links
+		    );
+		else
+		    total++;
+	    }
+	    /* Directory? */
+#ifdef S_ISDIR
+	    else if(S_ISDIR(m))
+#else
+	    else if(FALSE)
+#endif
+	    {
+		total += AVScanOPCalculateTotalFilesInDirIterate(
+		    path,
+		    recursive, executables_only, ignore_links
+		);
+	    }
+	    /* File? */
+#ifdef S_ISREG
+	    else if(S_ISREG(m))
+#else
+	    else if(TRUE)
+#endif
+	    {
+		total++;
+	    }
+	}
+
+	return(total);
+}
+
+
 /*
  *	Returns a dynamically allocated string describing the name and
  *	the version of the Antivirus Engine.
@@ -754,7 +1116,7 @@ gchar *AVScanGetAVEngineNameVersionString(void)
 #if defined(HAVE_CLAMAV)
 	const gchar	*config_prog = PREFIX "/bin/clamav-config",
 			*tmp_dir;
-	gint p;
+	gint p, fd;
 	gchar *cmd, *stdout_path, *version_str, *name_version_str;
 	FILE *fp;
 
@@ -772,7 +1134,9 @@ gchar *AVScanGetAVEngineNameVersionString(void)
 	    tmp_dir,
 	    PROG_NAME "XXXXXX"
 	));
-	mkstemp(stdout_path);
+	fd = (gint)mkstemp((char *)stdout_path);
+	if(fd > -1)
+	    close((int)fd);
 
 	cmd = g_strdup_printf(
 	    "\"%s\" --version",
@@ -825,19 +1189,20 @@ gchar *AVScanGetAVEngineNameVersionString(void)
  */
 gint AVScanOP(
 	const gchar *db_path,
-	GList *path_list,
-	gboolean recursive,
-	gboolean executables_only,
-	gint *stop_count
+	GList *paths_list,
+	const gboolean recursive,
+	const gboolean executables_only,
+	const gboolean ignore_links,
+	gint *stop_count		/* Must be preinitialized and not NULL */
 )
 {
 	gint status = 0;
-	gulong time_start = (gulong)time(NULL);
+	const gulong time_start = (gulong)time(NULL);
 #if defined(HAVE_CLAMAV)
 	ClamAVData *d;
 #endif
 
-	if(path_list == NULL)
+	if(paths_list == NULL)
 	{
 	    status = 2;
 	    return(status);
@@ -851,25 +1216,34 @@ gint AVScanOP(
 	}
 
 #if defined(HAVE_CLAMAV)
+	/* Initialize ClamAV */
 	d = AVScanOPClamAVInit(db_path);
 	if(d != NULL)
 	{
 	    gint status2;
 	    const gchar *path;
+	    mode_t m;
 	    GList *glist;
 	    struct stat stat_buf;
 
+	    /* Calculate the total files to be scanned */
+	    d->total_files = AVScanOPCalculateTotalFiles(
+		paths_list, recursive, executables_only, ignore_links
+	    );
+
 	    /* Iterate through each object in the paths list */
-	    for(glist = path_list; glist != NULL; glist = g_list_next(glist))
+	    for(glist = paths_list; glist != NULL; glist = g_list_next(glist))
 	    {
 		path = (const gchar *)glist->data;
 		if(STRISEMPTY(path))
 		    continue;
 
-		if(stat(path, &stat_buf))
+		/* Get this object's local stats */
+		if(lstat(path, &stat_buf))
 		{
-		    const gint errnum = (gint)errno;
-		    gchar *error_msg = STRDUP(g_strerror(errnum));
+		    /* Unable to get this object's local stats */
+		    const gint error_code = (gint)errno;
+		    gchar *error_msg = STRDUP(g_strerror((int)error_code));
 		    if(error_msg != NULL)
 		    {
 			*error_msg = toupper(*error_msg);
@@ -879,20 +1253,87 @@ gint AVScanOP(
 		    continue;
 		}
 
-#ifdef S_ISDIR
-		if(S_ISDIR(stat_buf.st_mode))
+		m = stat_buf.st_mode;
+
+		/* Link? */
+#ifdef S_ISLNK
+		if(S_ISLNK(m))
 #else
 		if(FALSE)
+#endif
+		{
+		    if(ignore_links)
+		    {
+			/* Print a warning about this because a link
+			 * was explicitly specified and ignore_links
+			 * was also specified
+			 */
+			PRINT_ERR(path, "Link ignored");
+			continue;
+		    }
+
+		    /* Get this link's destination stats */
+		    if(stat(path, &stat_buf))
+		    {
+			/* Unable to get this link's destination stats */
+			const gint error_code = (gint)errno;
+			gchar *error_msg = STRDUP(g_strerror((int)error_code));
+			if(error_msg != NULL)
+			{
+			    *error_msg = toupper(*error_msg);
+			    PRINT_ERR(path, error_msg);
+			    g_free(error_msg);
+			}
+			continue;
+		    }
+
+		    m = stat_buf.st_mode;
+
+#ifdef S_ISDIR
+		    if(S_ISDIR(m))
+#else
+		    if(FALSE)
+#endif
+			status2 = AVScanOPClamAVScanDir(
+			    d, path,
+			    recursive, executables_only, ignore_links,
+			    stop_count
+			);
+		    else
+			status2 = AVScanOPClamAVScanFile(
+			    d, path,
+			    stop_count
+			);
+		}
+		/* Directory? */
+#ifdef S_ISDIR
+		else if(S_ISDIR(m))
+#else
+		else if(FALSE)
 #endif
 		    status2 = AVScanOPClamAVScanDir(
-			d, path, recursive, executables_only,
+			d, path,
+			recursive, executables_only, ignore_links,
 			stop_count
 		    );
-		else
+		/* File? */
+#ifdef S_ISREG
+		else if(S_ISREG(m))
+#else
+		else if(TRUE)
+#endif
+		{
 		    status2 = AVScanOPClamAVScanFile(
 			d, path,
 			stop_count
 		    );
+		}
+		else
+		{
+		    PRINT_ERR(path, "Unsupported object type");
+		    status2 = 1;	/* Unsupported object type or nothing to scan */
+		}
+
 		if((status2 != 0) && (status == 0))
 		    status = status2;
 	    }
@@ -910,7 +1351,7 @@ gint AVScanOP(
 	    status = 3;
 	}
 #else
-	g_print("No AntiVirus support compiled\n");
+	g_print("No AntiVirus support enabled at compile time\n");
 	status = 1;
 #endif
 
