@@ -3,6 +3,7 @@
 #include <errno.h>
 #include <signal.h>
 #include <ctype.h>
+#include <time.h>
 #include <sys/stat.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
@@ -71,8 +72,7 @@ void WinScanProcessStart(
 	win_struct *win,
 	const gchar *db_location,	/* Virus Scanner Database Location */
 	const gchar *location,		/* Location of object to scan */
-	gboolean recursive,		/* Recursive? */
-	gboolean executables_only	/* Executables Only? */
+	const avscan_options options
 );
 void WinScanProcessStop(win_struct *win);
 gboolean WinScanProcessIsRunning(win_struct *win);
@@ -81,7 +81,7 @@ void WinUpdateProcessStop(win_struct *win);
 gboolean WinUpdateProcessIsRunning(win_struct *win);
 
 /* Scan List & Settings Update */
-void WinScanListUpdate(win_struct *win);
+void WinScanListOpenFromFile(win_struct *win, const gchar *path);
 void WinScanSettingsUpdate(win_struct *win, const obj_struct *obj);
 void WinResultsLocationUpdate(win_struct *win, const gchar *location);
 
@@ -112,11 +112,14 @@ void WinToolBarUpdateItems(win_struct *win);
 gboolean WinStatusBarIsShown(win_struct *win);
 void WinStatusBarSetShow(win_struct *win, gboolean show);
 void WinStatusProgress(
-	win_struct *win, gfloat val, gboolean allow_gtk_iteration
+	win_struct *win, const gfloat v, gboolean allow_gtk_iteration
 );
 void WinStatusMessage(
 	win_struct *win, const gchar *mesg, gboolean allow_gtk_iteration
 );
+void WinStatusTime(
+	win_struct *win, const gulong t, gboolean allow_gtk_iteration
+);
 gboolean WinIsMapped(win_struct *win);          
 void WinMap(win_struct *win);         
 void WinUnmap(win_struct *win);
@@ -184,10 +187,11 @@ void WinScanProcessStart(
 	win_struct *win,
 	const gchar *db_location,	/* Virus Scanner Database Location */
 	const gchar *location,		/* Location of object to scan */
-	gboolean recursive,		/* Recursive? */
-	gboolean executables_only	/* Executables Only? */
+	const avscan_options options
 )
 {
+	const gulong start_time = (gulong)time(NULL);
+	gint fd;
 	const gchar *tmp_dir;
 	gchar *cmd;
 	const cfg_item_struct *cfg_list;
@@ -210,6 +214,7 @@ void WinScanProcessStart(
 	    );
 	if(STRISEMPTY(db_location))
 	{
+	    /* Warn about the virus database location not being set */
 #ifdef HAVE_EDV2
 	    EDVPlaySoundWarning(core->edv_ctx); 
 #endif
@@ -284,31 +289,39 @@ To set the virus database location go to Database->Location...",
 	    tmp_dir,
 	    PROG_NAME "XXXXXX"
 	));
-	mkstemp(win->stdout_path);
+	fd = (gint)mkstemp((char *)win->stdout_path);
+	if(fd > -1)
+	    close((int)fd);
 	win->stderr_path = STRDUP(PrefixPaths(
 	    tmp_dir,
 	    PROG_NAME "XXXXXX"
 	));
-	mkstemp(win->stderr_path);
+	fd = (gint)mkstemp((char *)win->stderr_path);
+	if(fd > -1)
+	    close((int)fd);
 
 	/* Format scan command */
 	cmd = g_strdup_printf(
-	    "\"%s\" --scan \"%s\"%s%s --database \"%s\"",
+	    "\"%s\" --scan \"%s\"%s%s%s --database \"%s\"",
 	    core->prog_path, location,
-	    recursive ? " --recursive" : "",
-	    executables_only ? " --executables-only" : "",
+	    (options & AVSCAN_OPT_RECURSIVE) ?
+		" --recursive" : "",
+	    (options & AVSCAN_OPT_EXECUTABLES_ONLY) ?
+		" --executables-only" : "",
+	    (options & AVSCAN_OPT_IGNORE_LINKS) ?
+		" --ignore-links" : "",
 	    db_location
 	);
 
-	/* Run scan command */
+	/* Run the scan command */
 	win->scan_pid = (gint)ExecOE(
 	    cmd, win->stdout_path, win->stderr_path
 	);
 
-	/* Delete command */
+	/* Delete the scan command */
 	g_free(cmd);
 
-	/* Failed to run scan command? */
+	/* Failed to run the scan command? */
 	if(win->scan_pid <= 0)
 	{
 	    char *s = g_strdup_printf(
@@ -337,6 +350,9 @@ To set the virus database location go to Database->Location...",
 	    return;
 	}
 
+	/* Record the start time */
+	win->scan_start_time = start_time;
+
 	/* Open output files for reading */
 	win->stdout_fp = FOpen(win->stdout_path, "rb");
 	win->stderr_fp = FOpen(win->stderr_path, "rb");
@@ -373,14 +389,22 @@ void WinScanProcessStop(win_struct *win)
 	    win->scan_pid = 0;
 	}
 
+	/* Reset the scanned files and total files */
+	win->scan_files_scanned = 0l;
+	win->scan_files_total = 0l;
+
+	/* Reset the start time */
+	win->scan_start_time = 0l;
+
 	/* Close and delete the stdout & stderr files */
 	WinProcessOutputDelete(win);
 
 	/* Reset stop count */
 	win->stop_count = 0;
 
-	/* Reset progress */
+	/* Reset progress and time */
 	WinStatusProgress(win, 0.0f, FALSE);
+	WinStatusTime(win, 0l, FALSE);
 
 	WinSetPassiveBusy(win, FALSE);
 
@@ -404,7 +428,8 @@ gboolean WinScanProcessIsRunning(win_struct *win)
  */
 void WinUpdateProcessStart(win_struct *win)
 {
-	gint effective_uid = (gint)geteuid();
+	gint fd;
+	const gint effective_uid = (gint)geteuid();
 #if defined(HAVE_CLAMAV)
 	const gchar *prog = PREFIX "/bin/freshclam";
 #else
@@ -522,12 +547,16 @@ Do you want to try and update the virus database anyway?",
 	    tmp_dir,
 	    PROG_NAME "XXXXXX"
 	));
-	mkstemp(win->stdout_path);
+	fd = (gint)mkstemp((char *)win->stdout_path);
+	if(fd > -1)
+	    close((int)fd);
 	win->stderr_path = STRDUP(PrefixPaths(
 	    tmp_dir,
 	    PROG_NAME "XXXXXX"
 	));
-	mkstemp(win->stderr_path);
+	fd = (gint)mkstemp((char *)win->stderr_path);
+	if(fd > -1)
+	    close((int)fd);
 
 	/* Format virus database update command */
 	cmd = g_strdup_printf(
@@ -610,8 +639,9 @@ void WinUpdateProcessStop(win_struct *win)
 	/* Reset stop count */
 	win->stop_count = 0;
 
-	/* Reset progress */
+	/* Reset progress and time */
 	WinStatusProgress(win, 0.0f, FALSE);
+	WinStatusTime(win, 0l, FALSE);
 
 	WinSetPassiveBusy(win, FALSE);
 
@@ -629,11 +659,10 @@ gboolean WinUpdateProcessIsRunning(win_struct *win)
 
 
 /*
- *	Updates the scan list.
+ *	Opens the scan list from file and updates the scan list.
  */
-void WinScanListUpdate(win_struct *win)
+void WinScanListOpenFromFile(win_struct *win, const gchar *path)
 {
-	gchar *path;
 	GList *glist, *obj_list;
 	GtkCList *clist;
 #ifdef HAVE_EDV2
@@ -657,12 +686,8 @@ void WinScanListUpdate(win_struct *win)
 	/* Clear the scan list */
 	gtk_clist_clear(clist);
 
-	/* Load scan list from file */
-	path = STRDUP(PrefixPaths(
-	    core->prog_data_dir, AVSCAN_SCAN_LIST_FILE
-	));
+	/* Open the scan list from file */
 	obj_list = ObjListOpen(path, NULL, NULL);
-	g_free(path);
 
 	/* Transfer each object to the Win's scan list */
 	for(glist = obj_list; glist != NULL; glist = g_list_next(glist))
@@ -682,6 +707,7 @@ void WinScanListUpdate(win_struct *win)
  */
 void WinScanSettingsUpdate(win_struct *win, const obj_struct *obj)
 {
+	gboolean b;
 	GtkWidget *w;
 
 	if(win == NULL)
@@ -698,28 +724,37 @@ void WinScanSettingsUpdate(win_struct *win, const obj_struct *obj)
 	    );
 	}
 
+	b = (obj != NULL) ?
+	    (obj->options & AVSCAN_OPT_RECURSIVE) : FALSE;
 	GUIMenuItemSetCheck(
-	    win->recursive_micheck,
-	    (obj != NULL) ? obj->recursive : FALSE,
-	    TRUE
+	    win->recursive_micheck, b, TRUE
 	);
 	w = win->recursive_check;
 	if(w != NULL)
 	    gtk_toggle_button_set_active(
-		GTK_TOGGLE_BUTTON(w),
-		(obj != NULL) ? obj->recursive : FALSE
+		GTK_TOGGLE_BUTTON(w), b
 	    );
 
+	b = (obj != NULL) ?
+	    (obj->options & AVSCAN_OPT_EXECUTABLES_ONLY) : FALSE;
 	GUIMenuItemSetCheck(
-	    win->executables_only_micheck,
-	    (obj != NULL) ? obj->executables_only : FALSE,
-	    TRUE
+	    win->executables_only_micheck, b, TRUE
 	);
 	w = win->executables_only_check;
 	if(w != NULL)
 	    gtk_toggle_button_set_active(
-		GTK_TOGGLE_BUTTON(w),
-		(obj != NULL) ? obj->executables_only : FALSE
+		GTK_TOGGLE_BUTTON(w), b
+	    );
+
+	b = (obj != NULL) ?
+	    (obj->options & AVSCAN_OPT_IGNORE_LINKS) : FALSE;
+	GUIMenuItemSetCheck(
+	    win->ignore_links_micheck, b, TRUE
+	);
+	w = win->ignore_links_check;
+	if(w != NULL)
+	    gtk_toggle_button_set_active(
+		GTK_TOGGLE_BUTTON(w), b
 	    );
 
 	win->freeze_count--;
@@ -1059,6 +1094,18 @@ GList *WinOPIDListNew(win_struct *win)
 	func_cb = WinExecutablesOnlyCB;
 	DO_APPEND(WIN_OPID_EXECUTABLES_ONLY)
 
+	allow_multiple = FALSE;
+	item_type = TOOLBAR_ITEM_TOGGLE_BUTTON;
+	button_name = "IgLnks";
+	menu_name = "Ignore Links";
+	tooltip = "Toggle exclusion of scanning links";
+	accel_key = 0;
+	accel_mods = 0;
+	button_icon_data = NULL;
+	menu_icon_data = button_icon_data;                         
+	func_cb = WinIgnoreLinksCB;
+	DO_APPEND(WIN_OPID_IGNORE_LINKS)
+
 
 	allow_multiple = FALSE;
 	item_type = TOOLBAR_ITEM_BUTTON;
@@ -1536,6 +1583,10 @@ static void WinBuildMenuBar(win_struct *win)
 	    icon_data = NULL;
 	    DO_ADD_MENU_ITEM_CHECK(WIN_OPID_EXECUTABLES_ONLY)
 	    win->executables_only_micheck = w;
+
+	    icon_data = NULL;
+	    DO_ADD_MENU_ITEM_CHECK(WIN_OPID_IGNORE_LINKS)
+	    win->ignore_links_micheck = w;
 	}
 	win->scan_mlabel = GUIMenuAddToMenuBar(
 	    menubar, menu, "Scan", GUI_MENU_BAR_ALIGN_LEFT
@@ -2134,6 +2185,14 @@ static void WinBuildLists(win_struct *win)
 	    WinEntryDragDataReceivedCB,
 	    win
 	);
+	gtk_signal_connect(
+	    GTK_OBJECT(w), "key_press_event",
+	    GTK_SIGNAL_FUNC(WinEntryCompletePathKeyCB), win
+	);
+	gtk_signal_connect(
+	    GTK_OBJECT(w), "key_release_event",
+	    GTK_SIGNAL_FUNC(WinEntryCompletePathKeyCB), win
+	);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "activate",
 	    GTK_SIGNAL_FUNC(WinStartCB), win
@@ -2168,6 +2227,16 @@ static void WinBuildLists(win_struct *win)
 	);
 	gtk_widget_show(w);
 
+	win->ignore_links_check = w = gtk_check_button_new_with_label(
+	    "Ignore Links"
+	);
+	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
+	SET_CROSSING(WIN_OPID_EXECUTABLES_ONLY, w);
+	GUISetWidgetTip(w,
+"Check this to exclude links from the scan"
+	);
+	gtk_widget_show(w);
+
 
 	/* Results page tab */
 	tablabel = w = gtk_hbox_new(FALSE, border_minor);
@@ -2539,6 +2608,7 @@ static void WinBuildLists(win_struct *win)
  */
 static void WinBuildStatusBar(win_struct *win)
 {
+	const gint border_minor = 2;
 	gint attach_x = 0;
 	GtkAdjustment *adj;
 	GtkWidget *w, *parent, *parent2, *parent3;
@@ -2557,8 +2627,10 @@ static void WinBuildStatusBar(win_struct *win)
 	parent = w;
 
 	/* Main table */
-	w = gtk_table_new(1, 2, FALSE);
+	w = gtk_table_new(1, 3, FALSE);
 	gtk_container_add(GTK_CONTAINER(parent), w);
+	gtk_table_set_row_spacings(GTK_TABLE(w), border_minor);
+	gtk_table_set_col_spacings(GTK_TABLE(w), border_minor);
 	gtk_widget_show(w);
 	parent = w;
 
@@ -2593,7 +2665,6 @@ static void WinBuildStatusBar(win_struct *win)
 	/* Label */
 	w = gtk_frame_new(NULL);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_IN);
-	gtk_container_border_width(GTK_CONTAINER(w), 1);
 	gtk_table_attach(
 	    GTK_TABLE(parent), w,
 	    attach_x, attach_x + 1,
@@ -2613,7 +2684,34 @@ static void WinBuildStatusBar(win_struct *win)
 
 	win->statusbar_label = w = gtk_label_new("");
 	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
-	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 2);
+	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, border_minor);
+	gtk_widget_show(w);
+
+
+	/* Time Label */
+	w = gtk_frame_new(NULL);
+	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_IN);
+	gtk_widget_set_usize(w, 60, -1);
+	gtk_table_attach(
+	    GTK_TABLE(parent), w,
+	    attach_x, attach_x + 1,
+	    0, 1,
+	    0,
+	    GTK_FILL,
+	    0, 0
+	);
+	attach_x++;
+	gtk_widget_show(w);
+	parent2 = w;
+
+	w = gtk_hbox_new(FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(parent2), w);
+	gtk_widget_show(w);
+	parent3 = w;
+
+	win->statusbar_time_label = w = gtk_label_new("");
+	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
+	gtk_box_pack_end(GTK_BOX(parent3), w, FALSE, FALSE, border_minor);
 	gtk_widget_show(w);
 }
 
@@ -2808,6 +2906,9 @@ win_struct *WinNew(gpointer core_ptr)
 	win->db_ctree_load_idleid = 0;
 	win->scan_toid = 0;
 	win->scan_pid = 0;
+	win->scan_files_scanned = 0l;
+	win->scan_files_total = 0l;
+	win->scan_start_time = 0l;
 	win->update_toid = 0;
 	win->update_pid = 0;
 	win->stdout_fp = NULL;
@@ -3438,9 +3539,10 @@ void WinStatusBarSetShow(win_struct *win, gboolean show)
  *	Sets the Win's status bar progress.
  */
 void WinStatusProgress(
-	win_struct *win, gfloat val, gboolean allow_gtk_iteration
+	win_struct *win, const gfloat v, gboolean allow_gtk_iteration
 )
 {
+	gfloat val = v;
 	GtkProgress *progress;
 	GtkWidget *w = (win != NULL) ? win->statusbar_progress : NULL;
 	if(w == NULL)
@@ -3531,6 +3633,38 @@ void WinStatusMessage(
 	    gtk_main_iteration();
 }
 
+/*
+ *	Set's the Win's status bar time.
+ */
+void WinStatusTime(
+	win_struct *win, const gulong t, gboolean allow_gtk_iteration
+)
+{
+	gchar *s;
+	GtkWidget *w = (win != NULL) ? win->statusbar_time_label : NULL;
+	if(w == NULL)
+	    return;
+
+	if(t >= 3600l)
+	    s = g_strdup_printf(
+		"%i:%.2i:%.2i",
+		(gint)(t / 3600l),
+		(gint)((t / 60) % 60l),
+		(gint)(t % 60l)
+	    );
+	else
+	    s = g_strdup_printf(
+		"%.2i:%.2i",
+		(gint)(t / 60l),
+		(gint)(t % 60l)
+	    );
+	gtk_label_set_text(GTK_LABEL(w), s);
+	free(s);
+
+	while((gtk_events_pending() > 0) && allow_gtk_iteration)
+	    gtk_main_iteration();
+}
+
 /*
  *	Checks if the Win is mapped.
  */
@@ -3613,6 +3747,8 @@ void WinDelete(win_struct *win)
 	win->recursive_check = NULL;
 	GTK_WIDGET_DESTROY(win->executables_only_check);
 	win->executables_only_check = NULL;
+	GTK_WIDGET_DESTROY(win->ignore_links_check);
+	win->ignore_links_check = NULL;
 	GTK_WIDGET_DESTROY(win->results_location_label);
 	win->results_location_label = NULL;
 	GTK_WIDGET_DESTROY(win->results_clist);
