@@ -1,3 +1,4 @@
+#include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <sys/stat.h>
@@ -25,7 +26,14 @@
 #include "images/icon_biohazard_32x32.xpm"
 
 
+static const gchar *SEEK_PAST_ARG(const gchar *s);
+static const gchar *SEEK_PAST_SPACES(const gchar *s);
+static const gchar *GET_ARG(
+	const gchar *s, const gchar end_c, 
+	gchar **arg_rtn
+);
 static const gchar *GET_NAME_FROM_PATH(const gchar *path);
+static gchar *SHORTEN_STRING(const gchar *s, const gint m);
 
 void WinScanListItemDestroyCB(gpointer data);
 void WinResultsListItemDestroyCB(gpointer data);
@@ -56,6 +64,10 @@ void WinNotebookSwitchPageCB(
 	gpointer data
 );
 
+gint WinEntryCompletePathKeyCB(
+	GtkWidget *widget, GdkEventKey *key, gpointer data
+);
+
 gint WinListEventCB(GtkWidget *widget, GdkEvent *event, gpointer data);
 void WinListResizeColumnCB(
 	GtkCList *clist, gint column, gint width, gpointer data
@@ -99,18 +111,87 @@ gint WinScanMonitorTOCB(gpointer data);
 gint WinUpdateMonitorTOCB(gpointer data);
 
 
-#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
-#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
-#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
-#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)
+#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)
 
-#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
-#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
-#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
-#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
-#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
+#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
+#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
+#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
+#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
+#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
+/*
+ *	Seeks past non-blank characters until a blank character or
+ *	null character is encountered.
+ */
+static const gchar *SEEK_PAST_ARG(const gchar *s)
+{
+	if(s == NULL)
+	    return(NULL);
+
+	while(!ISBLANK(*s) && (*s != '\0'))
+	    s++;
+
+	return(s);
+}
+
+/*
+ *	Seeks past blank characters.
+ */
+static const gchar *SEEK_PAST_SPACES(const gchar *s)
+{
+	if(s == NULL)
+	    return(NULL);
+
+	while(ISBLANK(*s))
+	    s++;
+
+	return(s);
+}
+
+/*
+ *	Gets a dynamically allocated string *arg_rtn containing the
+ *	string from s to the next encountered character specified by
+ *	end_c.
+ *
+ *	If end_c is not encountered then a copy of s is returned as
+ *	*arg_rtn.
+ *
+ *	Returns the pointer in s past end_c if end_c was encountered or
+ *	NULL if end_c was not encountered.
+ */
+static const gchar *GET_ARG(
+	const gchar *s, const gchar end_c,
+	gchar **arg_rtn
+)
+{
+	gint len;
+	const gchar *s_end;
+
+	if(arg_rtn != NULL)
+	    *arg_rtn = NULL;
+
+	if(s == NULL)
+	    return(NULL);
+
+	s_end = strchr(s, end_c);
+	len = (s_end != NULL) ? (s_end - s) : STRLEN(s);
+
+	if(arg_rtn != NULL)
+	{
+	    gchar *arg = (gchar *)g_malloc((len + 1) * sizeof(gchar));
+	    if(len > 0)
+		memcpy(arg, s, len * sizeof(gchar));
+	    arg[len] = '\0';
+	    *arg_rtn = arg;
+	}
+
+	return((s_end != NULL) ? (s_end + 1) : NULL);
+}
+
 /*
  *      Returns the name portion of the path (if any).
  */
@@ -125,6 +206,33 @@ static const gchar *GET_NAME_FROM_PATH(const gchar *path)
 	return((s != NULL) ? (s + 1) : path);
 }
 
+/*
+ *	Returns a copy of the string that is no longer than m
+ *	characters with appropriate shortened notation where
+ *	appropriate.
+ */
+static gchar *SHORTEN_STRING(const gchar *s, const gint m)
+{
+	gint len;
+
+	if(s == NULL)
+	    return(NULL);
+
+	len = STRLEN(s);
+	if((len > m) && (m > 3))
+	{
+	    /* Need to shorten string */
+	    const gint i = len - m + 3;
+	    return(g_strdup_printf(
+		"...%s", (const gchar *)(s + i)
+	    ));
+	}
+	else
+	{
+	    return(STRDUP(s));
+	}
+}
+
 
 /*
  *	Win Scan List item destroy signal callback.
@@ -183,12 +291,12 @@ gint WinEventCB(
 	    keyval = key->keyval;
 	    state = key->state;
 #define DO_STOP_KEY_SIGNAL_EMIT {               \
- gtk_signal_emit_stop_by_name(                  \
-  GTK_OBJECT(widget),                           \
+ gtk_signal_emit_stop_by_name(			\
+  GTK_OBJECT(widget),				\
   press ?                                       \
    "key_press_event" : "key_release_event"      \
- );                                             \
-}  
+ );						\
+}
 	    switch(keyval)
 	    {
 	      case GDK_Escape:
@@ -310,6 +418,67 @@ void WinNotebookSwitchPageCB(
 	}
 }
 
+
+/*
+ *	Entry complete path "key_press_event" or "key_release_event"
+ *	signal callback.
+ */
+gint WinEntryCompletePathKeyCB(
+        GtkWidget *widget, GdkEventKey *key, gpointer data
+)
+{
+        gint status = FALSE;
+        gboolean press;
+        GtkEntry *entry = GTK_ENTRY(widget);
+	win_struct *win = WIN(data);
+        if((entry == NULL) || (key == NULL) || (win == NULL))
+            return(status);
+
+        press = (key->type == GDK_KEY_PRESS) ? TRUE : FALSE;
+
+#define SIGNAL_EMIT_STOP	{		\
+ gtk_signal_emit_stop_by_name(			\
+  GTK_OBJECT(widget),				\
+  press ?					\
+   "key_press_event" : "key_release_event"	\
+ );						\
+}
+
+        switch(key->keyval)
+        {
+          case GDK_Tab:
+            if(press)
+            {
+                gchar *s = STRDUP(gtk_entry_get_text(entry));
+                if(s != NULL)
+                {
+                    gint status;
+
+                    s = CompletePath(s, &status);
+                    gtk_entry_set_text(entry, (s != NULL) ? s : "");
+                    gtk_entry_set_position(entry, -1);
+                    g_free(s);
+
+                    switch(status)
+                    {
+                      case COMPLETE_PATH_NONE:
+                      case COMPLETE_PATH_AMBIGUOUS:
+                        gdk_beep();
+                        break;
+                    }
+                }
+            }
+            SIGNAL_EMIT_STOP
+            status = TRUE;
+            break;
+        }
+
+#undef SIGNAL_EMIT_STOP
+
+        return(status);
+}
+
+
 /*
  *	Win list event signal callback.
  */
@@ -334,11 +503,11 @@ gint WinListEventCB(GtkWidget *widget, GdkEvent *event, gpointer data)
 	    keyval = key->keyval;
 	    state = key->state;
 #define DO_STOP_KEY_SIGNAL_EMIT {               \
- gtk_signal_emit_stop_by_name(                  \
-  GTK_OBJECT(widget),                           \
+ gtk_signal_emit_stop_by_name(			\
+  GTK_OBJECT(widget),				\
   press ?                                       \
    "key_press_event" : "key_release_event"      \
- );                                             \
+ );						\
 }
 	    switch(keyval)
 	    {
@@ -391,7 +560,7 @@ gint WinListEventCB(GtkWidget *widget, GdkEvent *event, gpointer data)
 			menu, NULL, NULL,
 			NULL, NULL,
 			button->button, button->time
-		    );   
+		    );
 		}
 		status = TRUE;
 		break;
@@ -403,7 +572,7 @@ gint WinListEventCB(GtkWidget *widget, GdkEvent *event, gpointer data)
 	    switch(button->button)
 	    {
 	      case 1:
-		if(widget == win->scan_clist)  
+		if(widget == win->scan_clist)
 		{  
 		    WinStartCB(NULL, win);
 		}
@@ -613,7 +782,7 @@ void WinListSelectRowCB(
 		    buf = g_strdup_printf(
 			"Object \"%s\" selected",
 			name
-		    );      
+		    );
 
 		WinStatusMessage(win, buf, FALSE);
 		g_free(buf);
@@ -725,7 +894,7 @@ void WinTreeExpandCB(
 {  
 	win_struct *win = WIN(data);
 	if((ctree == NULL) || (win == NULL))
-	    return;  
+	    return;
 
 	if(win->freeze_count > 0)
 	    return;
@@ -913,13 +1082,13 @@ gint WinDBTreeLoadStartIdleCB(gpointer data)
 {
 	win_struct *win = WIN(data);
 	if(win == NULL)
-	    return(FALSE);    
+	    return(FALSE);
 
 	/* Update the Virus Database Tree as needed */
 	if(win->db_root_node == NULL)
 	{
 	    WinDBUpdate(win, NULL, TRUE);
-	}    
+	}
 
 	win->db_ctree_load_idleid = 0;
 	WinUpdate(win);
@@ -1069,14 +1238,119 @@ gint WinScanMonitorTOCB(gpointer data)
 		while(ISBLANK(*s))
 		    s++;
 
-		/* Scanning file or directory? */
+		/* Scanning file? */
 		if(g_strcasepfx(s, "Scanning:") ||
-		   g_strcasepfx(s, "Scanning file:") || 
-		   g_strcasepfx(s, "Scanning directory:")
+		   g_strcasepfx(s, "Scanning file:")
 		)
 		{
-		    /* Print this line to the status bar's message */
-		    WinStatusMessage(win, s, TRUE);
+		    /* Format is:
+		     *
+		     * "<pfx>: "<path>" <files_scanned> <total_files>"
+		     */
+		    const gchar *s2 = s;
+		    gchar *pfx = NULL, *path = NULL, *msg;
+		    gulong i = 0l, n = 0l;
+
+		    /* Get the prefix */
+		    if(s2 != NULL)
+		    {
+			s2 = GET_ARG(s2, ':', &pfx);
+		    }
+
+		    /* Get the path */
+		    if(s2 != NULL)
+		    {
+			gchar *path2;
+
+			s2 = strchr(s2, '"');
+			if(s2 != NULL)
+			    s2++;
+			s2 = GET_ARG(s2, '"', &path);
+
+			/* Shorten the path as needed */
+			path2 = SHORTEN_STRING(path, 40);
+			g_free(path);
+			path = path2;
+		    }
+
+		    /* Get the number of files scanned */
+		    if(s2 != NULL)
+		    {
+			s2 = SEEK_PAST_SPACES(s2);
+			win->scan_files_scanned = i = (gulong)ATOL(s2);
+		    }
+
+		    /* Get the total files */
+		    if(s2 != NULL)
+		    {
+			s2 = SEEK_PAST_ARG(s2);
+			s2 = SEEK_PAST_SPACES(s2);
+			win->scan_files_total = n = (gulong)ATOL(s2);
+		    }
+
+		    msg = g_strdup_printf(
+			"%s (%ld/%ld): %s",
+			pfx, i + 1l, n, path
+		    );
+		    WinStatusMessage(win, msg, FALSE);
+		    g_free(msg);
+
+		    WinStatusProgress(
+			win,
+			(n > 0l) ? ((gfloat)i / (gfloat)n) : 0.0f,
+			TRUE
+		    );
+
+		    g_free(path);
+		    g_free(pfx);
+		}
+		/* Scanning directory? */
+		else if(g_strcasepfx(s, "Scanning directory:"))
+		{
+#if 0
+/* We ignore the scanning directory because it usually appears very
+ * brief and makes subsequent messages difficult to read in the cases
+ * of scanning multiple subdirectories
+ */
+		    /* Format is:
+		     *
+		     * "<pfx>: "<path>""
+		     */
+		    const gchar *s2 = s;
+		    gchar *pfx = NULL, *path = NULL, *msg;
+
+		    /* Get the prefix */
+		    if(s2 != NULL)
+		    {
+			s2 = GET_ARG(s2, ':', &pfx);
+		    }
+
+		    /* Get the path */
+		    if(s2 != NULL)
+		    {
+			gchar *path2;
+
+			s2 = strchr(s2, '"');
+			if(s2 != NULL)
+			    s2++;
+			s2 = GET_ARG(s2, '"', &path);
+
+			/* Shorten the path as needed */
+			path2 = SHORTEN_STRING(path, 45);
+			g_free(path);
+			path = path2;
+		    }
+
+		    msg = g_strdup_printf(
+			"%s: %s",
+			pfx, path
+		    );
+		    WinStatusMessage(win, msg, TRUE);
+		    g_free(msg);
+
+		    g_free(path);
+		    g_free(pfx);
+#endif
 		}
 		/* Report */
 		else if(g_strcasepfx(s, "Report:"))
@@ -1102,8 +1376,8 @@ gint WinScanMonitorTOCB(gpointer data)
 		    /* Raise Win */
 		    WinMap(win);
 
-		    WinStatusMessage(win, "Scan done", TRUE);
-		    WinStatusProgress(win, 0.0f, TRUE);
+		    WinStatusProgress(win, 1.0f, FALSE);
+		    WinStatusMessage(win, "Scan done", FALSE);
 
 		    /* Display message in dialog */
 		    if(!CDialogIsQuery())
@@ -1142,10 +1416,10 @@ gint WinScanMonitorTOCB(gpointer data)
 		    EDVPlaySoundWarning(core->edv_ctx);
 #endif
 		    /* Raise Win */
-		    WinMap(win);                
+		    WinMap(win);
 
-		    WinStatusMessage(win, "Scan done", TRUE);
-		    WinStatusProgress(win, 0.0f, TRUE);
+		    WinStatusProgress(win, 1.0f, FALSE);
+		    WinStatusMessage(win, "Scan done", FALSE);
 
 		    /* Display message in dialog */
 		    if(!CDialogIsQuery())
@@ -1179,10 +1453,24 @@ gint WinScanMonitorTOCB(gpointer data)
 	    WinStatusMessage(win, "Scan done", TRUE);
 
 	    return(FALSE);
-	}    
+	}
 
-	/* Update progress */
-	WinStatusProgress(win, -1.0f, FALSE);
+	/* Update the progress only if the total number of files has
+	 * not been received yet
+	 */
+	if(win->scan_files_total == 0l)
+	    WinStatusProgress(win, -1.0f, FALSE);
+
+	/* Update the time */
+	if(win->scan_start_time > 0l)
+	{
+	    const gulong t = (gulong)time(NULL);
+	    WinStatusTime(
+		win,
+		(t > win->scan_start_time) ? (t - win->scan_start_time) : 0l,
+		FALSE
+	    );
+	}
 
 	return(TRUE);
 }
@@ -1202,11 +1490,11 @@ gint WinUpdateMonitorTOCB(gpointer data)
 	core_struct *core;
 	win_struct *win = WIN(data);
 	if(win == NULL)
-	    return(FALSE);   
+	    return(FALSE);
 
 	p = win->update_pid;
 	toplevel = win->toplevel;
-	core = CORE(win->core);   
+	core = CORE(win->core);
 
 	/* Need to stop? */
 	if(win->stop_count > 0)
@@ -1245,7 +1533,7 @@ gint WinUpdateMonitorTOCB(gpointer data)
 
 	/* Read stderr */
 	fp = win->stderr_fp;
-	if(fp != NULL)     
+	if(fp != NULL)
 	{
 	    gchar *s;
 
