@@ -421,7 +421,7 @@ void WinScanListUpdateRow(
 	if(column < columns)
 	{
 	    time_t t = (time_t)((obj != NULL) ? obj->last_runned : 0l);
-	    const gchar *s = (t > 0) ? TIME_DURATION_STRING(
+	    const gchar *s = (t > 0l) ? TIME_DURATION_STRING(
 		core, cur_time - t, t
 	    ) : "Unknown";
 	    gtk_clist_set_text(
