@@ -59,6 +59,7 @@ void WinStopCB(GtkWidget *widget, gpointer data);
 void WinLocationCB(GtkWidget *widget, gpointer data);
 void WinRecursiveCB(GtkWidget *widget, gpointer data);
 void WinExecutablesOnlyCB(GtkWidget *widget, gpointer data);
+void WinIgnoreLinksCB(GtkWidget *widget, gpointer data);
 
 static gint WinEDVRecycleProgressCB(   
 	gpointer data, gulong pos, gulong total
@@ -310,8 +311,7 @@ void WinAddCB(GtkWidget *widget, gpointer data)
 	    /* Set default object values */
 	    obj->name = STRDUP("New Scan");
 	    obj->path = STRDUP(getenv("HOME"));
-	    obj->recursive = FALSE;
-	    obj->executables_only = TRUE;
+	    obj->options = 0;
 	    obj->last_runned = 0l;
 
 	    /* Insert or append a new row on the scan list and
@@ -446,11 +446,15 @@ void WinEditCB(GtkWidget *widget, gpointer data)
 	    NULL, "Location:", obj->path,
 	    win, WinPDialogBrowseLocationCB
 	);
+	PDialogSetPromptCompletePath(-1);
 	PDialogAddPromptToggle(
-	    NULL, "Recursive", obj->recursive
+	    NULL, "Recursive", obj->options & AVSCAN_OPT_RECURSIVE
 	);
 	PDialogAddPromptToggle(
-	    NULL, "Executables Only", obj->executables_only
+	    NULL, "Executables Only", obj->options & AVSCAN_OPT_EXECUTABLES_ONLY
+	);
+	PDialogAddPromptToggle(
+	    NULL, "Ignore Links", obj->options & AVSCAN_OPT_IGNORE_LINKS
 	);
 	if(TRUE)
 	{
@@ -463,7 +467,7 @@ void WinEditCB(GtkWidget *widget, gpointer data)
 	    const gchar *s = (t > 0l) ? ctime(&t) : "Unknown";
 #endif
 	    gchar *buf = g_strdup_printf(
-		"Last Runned: %s", s
+		"Last Scanned: %s", s
 	    );
 	    PDialogAddPromptLabel(buf);
 	    g_free(buf);
@@ -486,7 +490,7 @@ void WinEditCB(GtkWidget *widget, gpointer data)
 	    GTK_WINDOW(PDialogGetToplevel()), NULL
 	);
 /*	PDialogSetTransientFor(NULL); */
-	if((strv != NULL) && (strc > 3))
+	if((strv != NULL) && (strc > 4))
 	{
 	    g_free(obj->name);
 	    obj->name = STRDUP(strv[0]);
@@ -494,9 +498,20 @@ void WinEditCB(GtkWidget *widget, gpointer data)
 	    g_free(obj->path);
 	    obj->path = STRDUP(strv[1]);
 
-	    obj->recursive = ATOI(strv[2]) ? TRUE : FALSE;
+	    if(ATOI(strv[2]))
+		obj->options |= AVSCAN_OPT_RECURSIVE;
+	    else
+		obj->options &= ~AVSCAN_OPT_RECURSIVE;
 
-	    obj->executables_only = ATOI(strv[3]) ? TRUE : FALSE;
+	    if(ATOI(strv[3]))
+		obj->options |= AVSCAN_OPT_EXECUTABLES_ONLY;
+	    else
+		obj->options &= ~AVSCAN_OPT_EXECUTABLES_ONLY;
+
+	    if(ATOI(strv[4]))
+		obj->options |= AVSCAN_OPT_IGNORE_LINKS;
+	    else
+		obj->options &= ~AVSCAN_OPT_IGNORE_LINKS;
 
 	    WinScanListUpdateRow(win, row, obj);
 	    WinScanSettingsUpdate(win, obj);
@@ -894,7 +909,7 @@ void WinStartCB(GtkWidget *widget, gpointer data)
 	GtkWidget *toplevel;
 	GtkCList *clist;
 	gchar *location;
-	gboolean recursive, executables_only;
+	avscan_options options;
 	obj_struct *obj;
 	core_struct *core;
 	win_struct *win = WIN(data);
@@ -917,12 +932,13 @@ void WinStartCB(GtkWidget *widget, gpointer data)
 	location = STRDUP(
 	    gtk_entry_get_text(GTK_ENTRY(win->location_entry))
 	);
-	recursive = GTK_TOGGLE_BUTTON_GET_ACTIVE(
-	    win->recursive_check
-	);
-	executables_only = GTK_TOGGLE_BUTTON_GET_ACTIVE(
-	    win->executables_only_check
-	);
+	options = 0;
+	if(GTK_TOGGLE_BUTTON_GET_ACTIVE(win->recursive_check))
+	    options |= AVSCAN_OPT_RECURSIVE;
+	if(GTK_TOGGLE_BUTTON_GET_ACTIVE(win->executables_only_check))
+	    options |= AVSCAN_OPT_EXECUTABLES_ONLY;
+	if(GTK_TOGGLE_BUTTON_GET_ACTIVE(win->ignore_links_check))
+	    options |= AVSCAN_OPT_IGNORE_LINKS;
 
 	/* Scan location not specified? */
 	if(STRISEMPTY(location))
@@ -959,7 +975,7 @@ void WinStartCB(GtkWidget *widget, gpointer data)
 	    /* Object's location matches the specified location? */
 	    if(!strcmp(obj->path, location))
 	    {
-		/* Update object's last runned time */
+		/* Update this object's last runned time */
 		obj->last_runned = (gulong)time(NULL);
 		WinScanListUpdateRow(win, i, obj);
 	    }
@@ -970,8 +986,7 @@ void WinStartCB(GtkWidget *widget, gpointer data)
 	    win,
 	    NULL,		/* Database Path */
 	    location,		/* Path to object to scan */
-	    recursive,		/* Recursive */
-	    executables_only	/* Executables Only */
+	    options
 	);
 
 	/* Switch to results page */
@@ -1136,6 +1151,41 @@ void WinExecutablesOnlyCB(GtkWidget *widget, gpointer data)
 	WinSetBusy(win, FALSE);
 }
 
+/*
+ *	Ignore Links.
+ *
+ *	Toggles ignore links.
+ */
+void WinIgnoreLinksCB(GtkWidget *widget, gpointer data)
+{
+	gboolean v;
+	win_struct *win = WIN(data);
+	if(win == NULL)
+	    return;
+
+	if(win->freeze_count > 0)
+	    return;
+
+	if((widget != NULL) ? !GTK_WIDGET_SENSITIVE(widget) : FALSE)
+	    return;
+
+	win->freeze_count++;
+	WinSetBusy(win, TRUE);
+
+	v = GTK_TOGGLE_BUTTON_GET_ACTIVE(win->ignore_links_check) ? FALSE : TRUE;
+	GUIMenuItemSetCheck(
+	    win->ignore_links_micheck, v, TRUE
+	);
+	gtk_toggle_button_set_active(
+	    GTK_TOGGLE_BUTTON(win->ignore_links_check), v
+	);
+
+	WinUpdate(win);
+
+	win->freeze_count--;
+	WinSetBusy(win, FALSE);
+}
+
 
 /*
  *	Endeavour Recycle progress callback.
